use async_trait::async_trait;
use axum::extract::FromRequestParts;
use axum::http::request::Parts;
use axum::http::StatusCode;
use axum::RequestPartsExt;
use axum_extra::headers::authorization::{Authorization as ExtraAuthorization, Bearer};
use axum_extra::TypedHeader;
use tracing::info;

use crate::common::err::{AppError, AppErrorType};
use crate::common::jwt::JwtClaims;

#[derive(Debug, Clone)]
pub struct Authorization(pub JwtClaims);


#[async_trait]
impl<S> FromRequestParts<S> for Authorization {
    type Rejection = (StatusCode, AppError);

    async fn from_request_parts(parts: &mut Parts, _state: &S) -> Result<Self, Self::Rejection> {
        let TypedHeader(ExtraAuthorization(bearer)) = parts
            .extract::<TypedHeader<ExtraAuthorization<Bearer>>>()
            .await
            .map_err(|error| {
                (StatusCode::UNAUTHORIZED, AppError::from_err(Box::new(error), AppErrorType::Authorization))
            })?;

        info!("Authorization from_request_parts");

        let claims = JwtClaims::verify(bearer.token())
            .map_err(|error| (StatusCode::UNAUTHORIZED, error))?;

        Ok(Self(claims))
    }
}

